import "./plugins.ts";

import config from "./config.ts";
import { serveDir } from "@std/http";
import { normalize, resolve } from "@std/path/posix";
import { renderView } from "./views/resolution.ts";
import { loadPlugins } from "./plugins.ts";

export function list(data: string | Iterable<string> | null | undefined): string[] {
    if (Array.isArray(data)) return data.flatMap(list);
    else if (typeof data === "string") return String(data ?? "")
        .split(/[,;]/)
        .map(v => v.trim())
        .filter(v => v !== "");
    else return [...(data ?? [])].flatMap(list);
}

export default async function main() {
    await loadPlugins();

    Deno.addSignalListener("SIGINT", async () => {
        console.log("Stopping server");
        server.shutdown();
        Deno.exit();
    });

    const server = Deno.serve({ port: 8080 }, async req => {
        try {
            const url = new URL(req.url);

            if (url.pathname === "/.well-known/keepalive") {
                return new Response("awake and alive");
            }

            if (config.static != null) {
                const res = await serveDir(req, {
                    showIndex: false,
                    fsRoot: resolve(config.static),
                    showDotfiles: false,
                    quiet: true,
                });

                if (res.status !== 404) return res;
            }

            const path = normalize("/" + url.pathname);
            const view = (renderView(path, { params: url.searchParams }, true))?.text;
            if (view != null) return new Response(view, { headers: { "content-type": "text/html" } });

            if (config.notFound != null && req.method === "GET") {
                const view = (renderView(config.notFound, { params: url.searchParams }, true))?.text;
                if (view != null) return new Response(view, { headers: { "content-type": "text/html" }, status: 404 });
            }

            return new Response("Not found", { status: 404 });
        }
        catch (e) {
            console.error(String(e));
            if (config.debug) {
                return new Response("Internal error: " + String(e), { status: 500 });
            }
            else return new Response("Internal error");
        }
    });
}

if (import.meta.main) {
    main();
}


// const app = express();
// export const lib = { View, PugView, MarkdownView, JSView, list, merge };

// app.use('/.well-known/keepalive', (req, res) => void res.send("awake and alive"));

// if (config.static != null) app.use(express.static(config.static));

// if (config.views != null) app.get("/*", (req, res, next) => {
//     const path = normalize("/" + req.path);

//     // for (const prefix of config.ignore) {
//     //     if (path.startsWith(prefix)) {
//     //         return next();
//     //     }
//     // }

//     const view = View.render(path, { query: req.query, params: req.params }, true);

//     if (view == null) next();
//     // else if (view.cached) res.sendStatus(304);
//     else res.send(view.text);
// });

// if (config.views != null) app.use((req, res) => {
//     res.status(404);
//     if (req.method === "GET") {
//         console.log(config.notFound);
//         const view = View.render(config.notFound, { query: req.query, params: req.params }, true, true);

//         if (view == null) res.send("Not found :/");
//         // else if (view.cached) res.sendStatus(304);
//         else res.send(view.text);
//     }
//     else res.send("Not found :/");
// });
// else app.use((req, res) => {
//     res.sendStatus(404);
// });

// app.listen(8080, () => console.log("Listening!"));


// // Learn more at https://docs.deno.com/runtime/manual/examples/module_metadata#concepts
